import { isApiGameData, type ApiGameData } from "$lib/ApiGameData";
import { type GameData } from "$lib/GameData";
import { isListing } from "$lib/Listing";
import { Game } from "$lib/server/Game";
import { updateListing } from "$lib/server/modifyListing";
import {
	readListingById,
	ServerCollections,
	writeUpdatedListing,
} from "$lib/server/mongo";
import { getBody, getParam, getUser, ResourceId } from "$lib/server/requestTools";
import {
	badRequestResponse,
	conflictResponse,
	forbiddenResponse,
	notFoundResponse,
	singleResponse,
} from "$lib/server/responseBodies";
import { retry } from "$lib/server/retry";
import type { RequestHandler } from "@sveltejs/kit";

export const GET: RequestHandler = async ({ params }): Promise<Response> => {
	const id = getParam(params, ResourceId.Game);

	if (!id) {
		return badRequestResponse("missing gameid parameter");
	}

	const game = await readListingById(ServerCollections.Games, id, isListing<GameData>);

	if (!game) {
		return notFoundResponse();
	}

	return singleResponse(game);
};

export const PUT: RequestHandler = async ({
	locals,
	request,
	params,
}): Promise<Response> => {
	const id = getParam(params, ResourceId.Game);

	if (!id) {
		return badRequestResponse("missing gameid parameter");
	}

	const putItem = async (): Promise<[boolean, Response]> => {
		const gameListing = await readListingById(
			ServerCollections.Games,
			id,
			isListing<GameData>,
		);

		if (!gameListing) {
			return [false, notFoundResponse()];
		}

		const game = Game.from(gameListing.data);

		let body: ApiGameData | null;
		try {
			body = await getBody(request, isApiGameData);
		} catch (err) {
			return [false, badRequestResponse("malformed game data in body")];
		}

		if (!body) {
			return [false, badRequestResponse("missing game data in body")];
		}

		const user = getUser(locals);
		const userId = user.payload.sub;

		if (game.isStarted !== body.isStarted) {
			if (game.isStarted) {
				return [false, conflictResponse()];
			}

			if (game.players[0].id !== userId) {
				return [false, forbiddenResponse()];
			}

			game.start();
		} else {
			return [false, singleResponse(gameListing)];
		}

		const res = await writeUpdatedListing(
			ServerCollections.Games,
			updateListing(gameListing, game),
		);

		if (!res.acknowledged || res.modifiedCount === 0) {
			return [true, conflictResponse()];
		}

		return [false, singleResponse(gameListing)];
	};

	return await retry(putItem, 200);
};
